<?php
/**
 * @file
 * Form alters implemented by the og_sm_taxonomy module.
 */

/**
 * Alters the terms administration form by adding the Site titles to the terms.
 *
 * @param array $form
 *   The form structure.
 */
function _og_sm_taxonomy_form_taxonomy_overview_terms_alter_site_names(array &$form) {
  // Only when not within a Site context.
  if (og_sm_current_site()) {
    return;
  }

  foreach (element_children($form) as $key) {
    if (!preg_match('/^tid\:[0-9]+/', $key)) {
      continue;
    }

    $element = &$form[$key];

    $term = taxonomy_term_load($element['#term']['tid']);
    $site = og_sm_taxonomy_term_get_site($term);
    if (!$site) {
      continue;
    }

    $element['view']['#suffix'] = ' <small>(' . $site->title . ')</small>';
  }
}

/**
 * Alters the empty text link to add the first Site term for the vocabulary.
 *
 * @param array $form
 *   The form structure.
 */
function _og_sm_taxonomy_form_taxonomy_overview_terms_alter_empty_text(array &$form) {
  // Only when there is an empty text element.
  if (empty($form['#empty_text'])) {
    return;
  }

  // Only within a Site context.
  $site = og_sm_current_site();
  if (!$site) {
    return;
  }

  $vocabulary = $form['#vocabulary'];
  $form['#empty_text'] = t(
    'No terms available. <a href="@link">Add term</a>.',
    array(
      '@link' => url('group/node/' . $site->nid . '/admin/taxonomy/' . $vocabulary->machine_name . '/add'),
    )
  );
}

/**
 * Alters the confirmation form to order the terms alphabetically.
 *
 * @param array $form
 *   The form structure.
 *
 * @see og_sm_taxonomy_form_taxonomy_overview_terms_alter()
 */
function _og_sm_taxonomy_form_taxonomy_overview_terms_alter_order(&$form) {
  // Only for the confirmation form.
  if (!isset($form['reset_alphabetical'])) {
    return;
  }

  // Update the cancel link target.
  $form['actions']['cancel']['#href'] = current_path();

  $form['site'] = array(
    '#type' => 'value',
    '#value' => og_sm_current_site(),
  );

  // Custom submit handler.
  $form['#submit'] = array(
    'og_sm_taxonomy_form_taxonomy_overview_terms_alter_submit',
  );
}

/**
 * Submit handler so we can overwrite the redirect after term sort confirmation.
 *
 * @param array $form
 *   The form structure.
 * @param array $form_state
 *   The form state.
 *
 * @see _og_sm_taxonomy_form_taxonomy_overview_terms_alter()
 */
function og_sm_taxonomy_form_taxonomy_overview_terms_alter_submit($form, &$form_state) {
  $values = $form_state['values'];
  $site = $values['site'];

  // Custom query to order only the terms of the current Site.
  // Drupal db_update() does not support joins :(.
  $query = <<<EOT
    UPDATE
      {taxonomy_term_data} td
      JOIN {og_membership} ogm ON (td.tid = ogm.etid)
    SET
      td.`weight` = 0
    WHERE
      td.vid = :vid
      AND ogm.gid = :gid
      AND ogm.entity_type = :entity_type 
      AND ogm.group_type = :group_type
EOT;

  db_query($query, array(
    ':vid' => $values['vid'],
    ':gid' => $site->nid,
    ':entity_type' => 'taxonomy_term',
    ':group_type' => 'node',
  ));

  drupal_set_message(t(
    'Reset vocabulary %name to alphabetical order.',
    array('%name' => $form_state['values']['name'])
  ));
  watchdog(
    'taxonomy',
    'Reset vocabulary %name to alphabetical order for Site %site_name.',
    array(
      '%name' => $form_state['values']['name'],
      '%site_name' => $site->title,
    ),
    WATCHDOG_NOTICE
  );

  // Set the proper redirect.
  $form_state['redirect'] = current_path();
}

/**
 * Alter the taxonomy add form.
 *
 * Adds the is_new property to fix entityreference prepopulate access check.
 *
 * @param array $form
 *   The form structure.
 * @param array $form_state
 *   The form state.
 */
function _og_sm_taxonomy_form_taxonomy_form_term_alter_add(array $form, array &$form_state) {
  // Set the entity state to is_new so the entityreference_prepopulate module
  // access check is working properly.
  $form_state['term']->is_new = TRUE;
}

/**
 * Alter the taxonomy term form.
 *
 * Adds the option to delete terms based on the OG permissions.
 *
 * @param array $form
 *   The form structure.
 * @param array $form_state
 *   The form state.
 */
function _og_sm_taxonomy_form_taxonomy_form_term_alter_delete(array &$form, array &$form_state) {
  $term = $form_state['term'];

  // Alter the delete access check.
  $form['actions']['delete']['#access'] = og_sm_taxonomy_term_delete_access($term);

  // Alter the confirmation form targets.
  if (isset($form_state['confirm_delete'])) {
    $form['actions']['cancel']['#href'] = current_path();
    $form['#submit'][] = 'og_sm_taxonomy_form_taxonomy_form_term_alter_delete_submit';
  }
}

/**
 * Custom form submit handler to set the redirect target to the terms overview.
 *
 * @param array $form
 *   The form structure.
 * @param array $form_state
 *   The form state.
 */
function og_sm_taxonomy_form_taxonomy_form_term_alter_delete_submit(array $form, array &$form_state) {
  $term = $form_state['term'];
  $site = og_sm_taxonomy_term_get_site($term);
  $vocabulary = taxonomy_vocabulary_load($term->vid);
  $form_state['redirect'] = 'group/node/' . $site->nid . '/admin/taxonomy/' . $vocabulary->machine_name;
}
